from http.client import HTTPException

import socketio
from fastapi import FastAPI, APIRouter, HTTPException
from starlette.websockets import WebSocket

from server.model.data import Game
# Game state
from server.model.players import RandomPlayer
from server.ws import sio

game = Game()

# Server
app = FastAPI()
router = APIRouter()

# Create and mount SocketIO app
sio_asgi_app = socketio.ASGIApp(
    socketio_server=sio,
    other_asgi_app=app
)
app.add_route("/socket.io/", route=sio_asgi_app, methods=['GET', 'POST'])
app.add_websocket_route("/socket.io/", sio_asgi_app)


@router.get("/")
async def hello_world():
    return "Hello, gentle[wo]man"


@router.post("/join")
async def join(player_name: str):
    if game.add_player(RandomPlayer(player_name)  # TODO: Let user play
                       ):
        return "Welcome %s, %i players currently waiting!" % (player_name, len(game.players))
    else:
        raise HTTPException(status_code=403, detail=f"{player_name} already connected, choose another name.")


@router.post("/ready")
async def ready():
    pass


@router.post("/play")
async def play():
    pass


@app.websocket("/ws")
async def websocket_endpoint(websocket: WebSocket):
    await websocket.accept()
    while True:
        data = await websocket.receive_text()
        await websocket.send_text(f"Message text was: {data}")


app.include_router(router)

if __name__ == '__main__':
    import uvicorn

    uvicorn.run("app:app", host="0.0.0.0", port=9042, reload=True)
